import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JPanel;


public class Carte extends JPanel{
	
	//position du pointeur de souris sur la carte
	public int xMouse;
	public int yMouse;
	
	private ImageIcon fond = null;//carte de fond
	private Reseau res;//reseau routier
	
	private int hauteur;//hauteur de la carte
	private int largeur;//largeur de la carte

	private double zoom;
	
	private boolean affPoints=false;//affichage des points
	private boolean affArcs=false;//affichage des arcs
	
	//points de depart et d'arrive pour calculer le trajet
	private ImageIcon depart=null;
	private ImageIcon arrivee=null;
	private boolean affDepart=false;
	private boolean affArrivee=false;
	private MyPoint ptDepart=null;
	private MyPoint ptArrivee=null;
	
	//liste d'arc correspondant  un trajet
	private ArrayList<Arc> trajet=null;
	private boolean affTrajet=false;
	
	public Carte(){
		//Chargement du reseau a partir du fichier xml
		res=new MapReader("carte/region_belfort_streets.xml").loadxml();
		
		fond = new ImageIcon(getClass().getResource("carte/"+res.getCheminCarte()));
		
		depart = new ImageIcon("graphique/Depart.png");
		arrivee = new ImageIcon("graphique/Arrivee.png");
		
		zoom=1;
		largeur=(int)(fond.getIconWidth()*zoom);
		hauteur=(int)(fond.getIconHeight()*zoom);
		trajet=new ArrayList<Arc>();
		this.setSize(largeur, hauteur);
		this.setPreferredSize(new Dimension(largeur, hauteur));
		
		
		
		//listener pour la souris
		this.addMouseListener(new TraitementSouris());
		this.addMouseMotionListener(new TraitementMove());
	}
	
	public double getZoom(){
		return zoom;
	}
	public Reseau getReseau(){
		return res;
	}
	public MyPoint getDepart(){
		return ptDepart;
	}
	public MyPoint getArrivee(){
		return ptArrivee;
	}
	public boolean getAffDepart(){
		return affDepart;
	}
	public boolean getAffArrivee(){
		return affArrivee;
	}
	public double getEchelle(){
		return res.getEchelle();
	}
	//mise en place d'un nouveau trajet
	public void setTrajet(ArrayList<Arc> _trajet){
		trajet.clear();
		trajet=_trajet;
	}
	
	//surcharge de la fonction paint pour dessiner tous les elements de la carte
	public void paint(Graphics g){
		Graphics2D graph = (Graphics2D)g;

		//appliquer le zoom sur le graphique
		graph.scale(zoom, zoom);
		
		//dessiner la carte
		fond.paintIcon(this, graph, 0, 0);
		//graph.drawImage(fond.getImage(), 0, 0, this);
		
		//dessiner les arcs  partir du rseau
		if(affArcs)
			dessinerArcs(graph);
		
		//dessiner les points
		if(affPoints)
			dessinerPoints(graph);
		
		if(affTrajet){
			dessinerTrajet(graph);
		}
	
		//dessiner l'image du drapeau de depart
		if(affDepart)
			graph.drawImage(depart.getImage(), ptDepart.getX(), ptDepart.getY()-(int)(depart.getIconHeight()/zoom),(int)(depart.getIconWidth()/zoom),(int)(depart.getIconHeight()/zoom), this);
			//depart.paintIcon(this, graph, ptDepart.getX(), ptDepart.getY()-depart.getIconHeight());
		
		//dessiner l'image du drapeau d'arrivee
		if(affArrivee)
			graph.drawImage(arrivee.getImage(), ptArrivee.getX(), ptArrivee.getY()-(int)(arrivee.getIconHeight()/zoom),(int)(arrivee.getIconWidth()/zoom),(int)(arrivee.getIconHeight()/zoom), this);
			//arrivee.paintIcon(this, graph, ptArrivee.getX(), ptArrivee.getY()-arrivee.getIconHeight());
		
	}
	
	//dessiner les points du reseau sur un graphic
	private void dessinerPoints(Graphics2D graph){
		ArrayList<MyPoint> listPts=res.getListPoints();
		for(int i=0; i<listPts.size();i++){
			graph.setColor(Color.RED);
			graph.fillOval(listPts.get(i).getX()-6, listPts.get(i).getY()-6, 12, 12);
		}
	}
	
	//dessiner les arcs du reseau sur un graphic
	private void dessinerArcs(Graphics2D graph){
		ArrayList<Arc> listArcs=res.getListArcs();
		for(int i=0;i<listArcs.size();i++){
			MyPoint pt1=listArcs.get(i).getDepart();
			MyPoint pt2=listArcs.get(i).getArrivee();
			graph.setColor(Color.BLACK);
			graph.drawLine(pt1.getX(), pt1.getY(), pt2.getX(), pt2.getY());
		}
	}
	
	//dessiner le trajet sur un graphic
	private void dessinerTrajet(Graphics2D graph){
		for(int i=0;i<trajet.size();i++){
			MyPoint pt1=trajet.get(i).getDepart();
			MyPoint pt2=trajet.get(i).getArrivee();
			graph.setStroke(new BasicStroke(5));
			graph.setColor(Color.RED);
			graph.drawLine(pt1.getX(), pt1.getY(), pt2.getX(), pt2.getY());
		}
	}
	
	//changer le zoom sur la carte
	public void updateZoom(double _zoom) {
		zoom=_zoom;
		largeur=(int)(fond.getIconWidth()*zoom);
		hauteur=(int)(fond.getIconHeight()*zoom);
		this.setSize(largeur, hauteur);
		this.setPreferredSize(new Dimension(largeur, hauteur));
		repaint();
	}
	
	//afficher les points sur la carte
	public void afficherPoints() {
		affPoints=!affPoints;
		repaint();
	}

	//afficher les arcs sur la carte
	public void afficherArcs() {
		affArcs=!affArcs;
		repaint();
	}
	
	//afficher le trajet sur la carte
	public void afficherTrajet(boolean b) {
		affTrajet=b;
		repaint();
	}
	
	//afficher le drapeau d'arrive
	public void afficherArrivee(){
		affArrivee=!affArrivee;
		repaint();
	}
	
	//afficher le drapeau de dpart
	public void afficherDepart(){
		affDepart=!affDepart;
		repaint();
	}

	
	//traitement de la souris: mise en place du depart et de l arrivee
	private class TraitementSouris implements MouseListener{
		public void mouseClicked(MouseEvent e){
			int x=(int)(e.getX()/zoom);
			int y=(int)(e.getY()/zoom);
			
			if(res.getPointPlusProche(x,y)!=null){
				if(!affDepart){
					ptDepart = res.getPointPlusProche(x,y);
					affDepart=!affDepart;
				}
				else if(!affArrivee){
					ptArrivee =  res.getPointPlusProche(x,y);
					affArrivee=!affArrivee;
				}
				else{
					if(ptDepart.getNum()==res.getPointPlusProche(x,y).getNum()){
						affDepart=!affDepart;
						affArrivee=!affArrivee;
					}
					else if(ptArrivee.getNum()==res.getPointPlusProche(x,y).getNum()){
						affArrivee=!affArrivee;
					}
					else{
						ptArrivee = res.getPointPlusProche(x,y);
					}
				}
				repaint();
			}
		}
		public void mouseEntered(MouseEvent e){}
		public void mouseExited(MouseEvent e){}
		public void mousePressed(MouseEvent e){}
		public void mouseReleased(MouseEvent e){}
    }
	
	//traitement de la molette de souris sur la carte:zoom
	private class TraitementMove implements MouseMotionListener{
		public void mouseMoved(MouseEvent e) {
			xMouse=(int)(e.getX()/zoom);
			yMouse=(int)(e.getY()/zoom);
		}
		public void mouseDragged(MouseEvent e){}
	}

}
