
import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class Exportateur {
	private String chemin;
	private Reseau res=null;
	private Document document;
	
	Exportateur(String _chemin, Reseau _res){
		chemin=_chemin;
		res=_res;
	}
	
	//exporte le reseau dans un fichier xml avec des coordonnes en Lambert II
	public void exec(){
		try{
			//construction du document avec les informations du reseau
			DocumentBuilderFactory fabrique=DocumentBuilderFactory.newInstance();
			DocumentBuilder constructeur=fabrique.newDocumentBuilder();
			document=constructeur.newDocument();
			
			//Proprits du DOM
			document.setXmlVersion("1.0");
			document.setXmlStandalone(true);
			
			//Cration de l'arborescence du DOM
			//noeud <carte ville="" src=""> (racine)
			Element racine=document.createElement("carte");
			racine.setAttribute("ville", res.getNom());
			racine.setAttribute("src", res.getCheminCarte());
			
			//noeud <ville nom="">
			Element ville=document.createElement("ville");
			ville.setAttribute("nom",res.getNom());
			racine.appendChild(ville);
			
			//noeud <points>
			Element points=document.createElement("points");

			Convertisseur c=new Convertisseur();
			int nbpoints=res.getListPoints().size();
			for(int i=0;i<nbpoints;i++){
				//noeud <point num="" x="" y="">
				Element point=document.createElement("point");
				point.setAttribute("num",Integer.toString(res.getListPoints().get(i).getNum()));
				c.pxToLambert(res.getListPoints().get(i).getX(), res.getListPoints().get(i).getY(), res.getEchelle());
				point.setAttribute("x",Double.toString(c.getX()));
				point.setAttribute("y",Double.toString(c.getY()));
				points.appendChild(point);
			}
			ville.appendChild(points);
			
			//noeud <rues>
			Element rues=document.createElement("rues");
			int nbrues=res.getListRues().size();
			for(int i=0;i<nbrues;i++){
				//noeud <rue nom="" sens="">
				Element rue=document.createElement("rue");
				rue.setAttribute("nom",res.getListRues().get(i).getNom());
				rue.setAttribute("sens", Integer.toString(res.getListRues().get(i).getSens()));
				
				int nbptrues=res.getListRues().get(i).getListPts().size();
				for(int j=0;j<nbptrues;j++){
					//noeud <pt num="">
					Element pt=document.createElement("pt");
					pt.setAttribute("num", Integer.toString(res.getListRues().get(i).getListPts().get(j).getNum()));
					rue.appendChild(pt);
				}
				rues.appendChild(rue);
			}
			ville.appendChild(rues);
			
			document.appendChild(racine);
			
			//exportation en xml
			Exportation();
		}
		catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
			
	}
	
	//exporter le document en fichier xml
	private void Exportation(){
		try{
			//document source  exporter
			Source source=new DOMSource(document);
			//fichier qui va tre cr
			File file = new File (chemin);
			Result resultat=new StreamResult(chemin);
			//creation d'une fabrique de document
			TransformerFactory fabrique=TransformerFactory.newInstance();
			Transformer transformer=fabrique.newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT,"yes");
			transformer.setOutputProperty(OutputKeys.ENCODING,"ISO-8859-1");
			
			//exporter le document en xml
			transformer.transform(source,resultat);
		}
		catch (Exception e) {
			e.printStackTrace();			
		}
	}
}
