import java.io.*;
import java.util.ArrayList;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class MapReader {
	private String chemin=null;
	private Reseau res=null;
	
	public MapReader(){
		chemin="";
		res=null;
	}
	
	public MapReader(String _chemin){
		chemin=_chemin;
	}
	 
    public Reseau loadxml(){
	    try{
		    // cration d'une fabrique de documents
		    DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
		    
		    // cration d'un constructeur de documents
		    DocumentBuilder constructeur = fabrique.newDocumentBuilder();
		    
		    // lecture du contenu d'un fichier XML avec DOM
		    File xml = new File(chemin);
		    Document docxml = constructeur.parse(xml);
		    
		    //element racine: on recupere le nom du reseau
		    Element racine = docxml.getDocumentElement();
		    res=new Reseau(racine.getAttribute("ville"));
		    res.setCheminCarte(racine.getAttribute("src"));
		    
		    //chargement de la liste des points du reseau
		    NodeList nl = racine.getElementsByTagName("point");
		    for (int i = 0; i < nl.getLength(); ++i) {
		          Element pt = (Element) nl.item(i);
		          
		          int numPt=Integer.parseInt(pt.getAttribute("num"));
		          int xPt=(int)Float.parseFloat(pt.getAttribute("x"));
		          int yPt=(int)Float.parseFloat(pt.getAttribute("y"));
		          
		          res.addPoint(new MyPoint(numPt, xPt, yPt ));
		    }
		    
		    //chargement des rues du reseau
		    nl = racine.getElementsByTagName("rue");
		    for (int i = 0; i < nl.getLength(); ++i) {
		          Element rue = (Element) nl.item(i);
		          
		          String nomRue=rue.getAttribute("nom");
		          int sens=(int)Integer.parseInt(rue.getAttribute("sens"));
		          
		          ArrayList<MyPoint> listPts=new ArrayList<MyPoint>();
		          ArrayList<Arc> listArcs = new ArrayList<Arc>();
		          MyPoint ptemp = null;
		          //chargement des points et des arcs de chaque rue
		          NodeList child=rue.getElementsByTagName("pt");
		          for(int j = 0; j < child.getLength(); ++j){
		        	  Element pt = (Element)child.item(j);
		        	  int numPt = Integer.parseInt(pt.getAttribute("num"));
		        	  MyPoint p=res.getPointByNum(numPt);
		        	  if(p!=null){
			        	  if(j==0){
			        		  listPts.add(new MyPoint(p.getNum(),p.getX(),p.getY()));
			        	  }
			        	  else{
			        		  //cration de l'arc avec le point prcdent
			        		  Arc a=new Arc(ptemp,p);
			        		  //ajout de l'arc dans la liste d'arcs
			        		  listArcs.add(a);
			        		  //chargement de l'arc dans le rseau
			        		  res.addArc(a);
			        		  //ajout du point  la liste de points
			        		  listPts.add(new MyPoint(p.getNum(),p.getX(),p.getY()));
			        	  }
			        	  ptemp = p;
		        	  }
		          }
		          //chargement de la rue dans le reseau
		          res.addRue(new Rue(nomRue, sens, listPts, listArcs));
		    }
		    
	    }
	    catch(ParserConfigurationException pce){
		    System.out.println("Erreur de configuration du parseur DOM");
		    System.out.println("lors de l'appel  fabrique.newDocumentBuilder();");
	    }
	    catch(SAXException se){
		    System.out.println("Erreur lors du parsing du document");
		    System.out.println("lors de l'appel au construteur.parse(xml)");
	    }
	    catch(IOException ioe){
		    System.out.println("Erreur d'entre/sortie");
		    System.out.println("lors de l'appel au construteur.parse(xml)");
	    }
	    
	    return res;

    }
    
   
}

