import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import javax.swing.*;


public class Projet extends JFrame{
	private JButton zoomMoins;
	private JButton zoomPlus;
	private JButton zoomUn;
	private JButton affichePoints;
	private JButton afficheRues;
	private JButton creerXML;
	private Menu m;//menu de gauche
	JScrollPane jspCarte;
	private Carte map;
	
	Projet(){
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setTitle("Reseau routier");
		this.getContentPane().setLayout(new BorderLayout());

		//menu haut
		zoomMoins = new JButton("Zoom -");
		zoomMoins.setPreferredSize(new Dimension(87,20));
		zoomPlus = new JButton("Zoom +");
		zoomPlus.setPreferredSize(new Dimension(87,20));
		zoomUn = new JButton("Zoom 1:1");
		zoomUn.setPreferredSize(new Dimension(87,20));
		affichePoints = new JButton("Points");
		affichePoints.setPreferredSize(new Dimension(87,20));
		afficheRues = new JButton("Rues");
		afficheRues.setPreferredSize(new Dimension(87,20));
		creerXML=new JButton("Exporter Lambert II");
		creerXML.setPreferredSize(new Dimension(165,20));
		
		JPanel haut =new JPanel();
		haut.add(zoomMoins);
		haut.add(zoomPlus);
		haut.add(zoomUn);
		haut.add(affichePoints);
		haut.add(afficheRues);
		haut.add(creerXML);
		
		//mise en place des listeners sur les boutons
		zoomMoins.addActionListener(new TraitementZoom());
		zoomPlus.addActionListener(new TraitementZoom());
		zoomUn.addActionListener(new TraitementZoom());
		affichePoints.addActionListener(new TraitementAffichage());
		afficheRues.addActionListener(new TraitementAffichage());
		creerXML.addActionListener(new TraitementExportation());
		map=new Carte();
		m=new Menu(map);
		
		//affichage de la carte dans un ScrollPane
		jspCarte = new JScrollPane(map);
		jspCarte.setSize(600,600);
		jspCarte.setPreferredSize(new Dimension(600,600));
		//mise en place du listener pour le zoom
		map.addMouseWheelListener(new TraitementRollZoom());
		
		this.getContentPane().add(m, BorderLayout.WEST);
		this.getContentPane().add(jspCarte, BorderLayout.CENTER);
		this.getContentPane().add(haut, BorderLayout.NORTH);
		
		this.pack();
		this.setVisible(true);
		
	}
	
	public static void main(String[] args) {
		new Projet();
		
	}
	
	//traitement des zooms boutons
	public class TraitementZoom implements ActionListener{
        public void actionPerformed(ActionEvent e){
        	JViewport vp = (JViewport)jspCarte.getViewport();
			
        	//coordonnes du centre du ScrollPane
			int xCentre=jspCarte.getSize().width/2;
			int yCentre=jspCarte.getSize().height/2;

			//coordonnes du point de la carte se trouvant au centre du ScrollPane
			int _x=(int)((vp.getViewPosition().x+xCentre)/map.getZoom());
			int _y=(int)((vp.getViewPosition().y+yCentre)/map.getZoom());
			
        	if(e.getSource()==zoomMoins){    			
        		if(map.getZoom() > 0.3){
        			//diminution du zoom en conservant le centrage de la carte
        			map.updateZoom(map.getZoom()-0.2);
        			vp.setViewPosition(new Point((int)(_x*map.getZoom()-xCentre),(int)(_y*map.getZoom()-yCentre)));
        		}
        	}
    		else if(e.getSource()==zoomPlus){
    			if(map.getZoom() < 1.5){
        			//augmentation du zoom en conservant le centrage de la carte
    				map.updateZoom(map.getZoom()+0.2);
    				vp.setViewPosition(new Point((int)(_x*map.getZoom()-xCentre),(int)(_y*map.getZoom()-yCentre)));
    			}
    		}
    		else if(e.getSource()==zoomUn){
    			//mise  1 du zoom en conservant le centrage de la carte
    			map.updateZoom(1);
    			vp.setViewPosition(new Point((int)(_x*map.getZoom()-xCentre),(int)(_y*map.getZoom()-yCentre)));
        	}
        }
    }
	
	//traitement du zoom molette
	public class TraitementRollZoom implements MouseWheelListener{
		public void mouseWheelMoved(MouseWheelEvent e) {
			JViewport vp = (JViewport)jspCarte.getViewport();
			
			//coordonnes du centre du ScrollPane
			int xCentre=jspCarte.getSize().width/2;
			int yCentre=jspCarte.getSize().height/2;
			
			//coordonnes du pointeur de souris sur la carte
			int _x=map.xMouse;
			int _y=map.yMouse;
			
			if(e.getWheelRotation()>0){
	        	if(map.getZoom() > 0.2){
	        		//diminution du zoom en ajustant le centrage de la carte au pointeur de souris
	        		map.updateZoom(map.getZoom()-0.1);
	        		vp.setViewPosition(new Point((int)(_x*map.getZoom()-xCentre),(int)(_y*map.getZoom()-yCentre)));
	        	}
			}
			else{
				if(map.getZoom() < 1.5){
	        		//augmentation du zoom en ajustant le centrage de la carte au pointeur de souris
					map.updateZoom(map.getZoom()+0.1);
					vp.setViewPosition(new Point((int)(_x*map.getZoom()-xCentre),(int)(_y*map.getZoom()-yCentre)));
				}
			}
			
		}
		
	}
	
	//traitement des boutons d'affichage des points et du rseau
	public class TraitementAffichage implements ActionListener{
        public void actionPerformed(ActionEvent e){
        	if(e.getSource()==affichePoints){
    			map.afficherPoints();
    		}
    		else if(e.getSource()==afficheRues){
    			map.afficherArcs();
    		}
        }
    }
	
	public class TraitementExportation implements ActionListener{
		
		public void actionPerformed(ActionEvent e) {
				Exportateur ex=new Exportateur("carte/Reseau_LambertII.xml",map.getReseau());
				ex.exec();
				
				
		}
		
	}

	
}
